using System;
using System.Text.Json;
using System.Threading.Tasks;

using Windows.ApplicationModel.DataTransfer;

using Microsoft.UI;
using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Controls;
using Microsoft.UI.Xaml.Media;
using Microsoft.Web.WebView2.Core;

namespace TestProject;

/// <summary>
///  
/// </summary>
public sealed partial class MainPage : Page
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// ؽƮ ڽ Ʈ  귯
    /// </summary>
    private Brush textBoxDefaultBackgroundBrush;

    /// <summary>
    /// ؽƮ ڽ   귯
    /// </summary>
    private SolidColorBrush textBoxSuccessBackgroundBrush = new SolidColorBrush(Colors.Green);

    /// <summary>
    /// ؽƮ ڽ н  귯
    /// </summary>
    private SolidColorBrush textBoxFailureBackgroundBrush = new SolidColorBrush(Colors.Red);

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - MainPage()

    /// <summary>
    /// 
    /// </summary>
    public MainPage()
    {
        InitializeComponent();

        this.textBoxDefaultBackgroundBrush = this.urlTextBox.Background;

        Loaded                    += Page_Loaded;
        Unloaded                  += Page_Unloaded;
        this.navigateButton.Click += navigateButton_Click;
        this.copyHTMLButton.Click += copyHTMLButton_Click;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region  ε óϱ - Page_Loaded(sender, e)

    /// <summary>
    ///  ε óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private async void Page_Loaded(object sender, RoutedEventArgs e)
    {
        Loaded -= Page_Loaded;

        await this.webView2.EnsureCoreWebView2Async();

        this.webView2.CoreWebView2.NavigationStarting  += coreWebView2_NavigationStarting;
        this.webView2.CoreWebView2.NavigationCompleted += coreWebView2_NavigationCompleted;

        this.webView2.Source = new Uri(this.urlTextBox.Text.Trim());
    }

    #endregion
    #region  ε óϱ - Page_Unloaded(sender, e)

    /// <summary>
    ///  ε óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void Page_Unloaded(object sender, RoutedEventArgs e)
    {
        if(this.webView2 != null)
        {
            this.webView2.Close();

            this.webView2 = null;
        }
    }

    #endregion
    #region ھ  2 Ž ۽ óϱ - coreWebView2_NavigationStarting(sender, e)

    /// <summary>
    /// ھ  2 Ž ۽ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void coreWebView2_NavigationStarting(CoreWebView2 sender, CoreWebView2NavigationStartingEventArgs e)
    {
        this.urlTextBox.IsEnabled     = false;
        this.navigateButton.IsEnabled = false;
        this.copyHTMLButton.IsEnabled = false;
    }

    #endregion
    #region ھ  2 Ž Ϸ óϱ - coreWebView2_NavigationCompleted(sender, e)

    /// <summary>
    /// ھ  2 Ž Ϸ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void coreWebView2_NavigationCompleted(CoreWebView2 sender, CoreWebView2NavigationCompletedEventArgs e)
    {
        if(e.IsSuccess)
        {
            this.urlTextBox.Background = this.textBoxSuccessBackgroundBrush;
        }
        else
        {
            this.urlTextBox.Background = this.textBoxFailureBackgroundBrush;
        }

        this.urlTextBox.IsEnabled     = true;
        this.navigateButton.IsEnabled = true;
        this.copyHTMLButton.IsEnabled = true;

        DispatcherQueue.TryEnqueue
        (
            async () =>
            {
                await Task.Delay(500);

                this.urlTextBox.Background = this.textBoxDefaultBackgroundBrush;

                this.urlTextBox.Focus(FocusState.Programmatic);

                this.urlTextBox.SelectAll();
            }
        );
    }

    #endregion
    #region Ž ư Ŭ óϱ - navigateButton_Click(sender, e)

    /// <summary>
    /// Ž ư Ŭ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private void navigateButton_Click(object sender, RoutedEventArgs e)
    {
        this.webView2.Source = new Uri(this.urlTextBox.Text.Trim());
    }

    #endregion
    #region HTML  ư Ŭ óϱ - copyTextButton_Click(sender, e)

    /// <summary>
    /// HTML  ư Ŭ óϱ
    /// </summary>
    /// <param name="sender">̺Ʈ ߻</param>
    /// <param name="e">̺Ʈ </param>
    private async void copyHTMLButton_Click(object sender, RoutedEventArgs e)
    {
        string html = await this.webView2.CoreWebView2.ExecuteScriptAsync("document.documentElement.outerHTML");

        html = JsonSerializer.Deserialize<string>(html);

        DataPackage dataPackage = new DataPackage();

        dataPackage.SetText(html);

        Clipboard.SetContent(dataPackage);
    }

    #endregion
}